'use strict';

var Promise = require('bluebird');
var Errors = require('common-errors');
var redisKey = require('../utils/key.js');
var emailVerification = require('../utils/send-email.js');
var jwt = require('../utils/jwt.js');

module.exports = function verifyChallenge(opts) {
  // TODO: add security logs
  // var remoteip = opts.remoteip;
  var token = opts.token;
  var audience = opts.audience;
  var namespace = opts.namespace;
  var username = opts.username;
  var redis = this.redis;
  var config = this.config;

  var promise = Promise.bind(this);

  if (!username) {
    promise = promise.then(function verifyToken() {
      return emailVerification.verify.call(this, token, namespace, config.validation.ttl > 0);
    });
  } else {
    promise = promise.then(function verifyUserExists() {
      var userKey = redisKey(username, 'data');
      return redis.hexists(userKey, 'active').then(function fieldExists(exists) {
        if (!exists) {
          throw new Errors.HttpStatusError(404, 'user does not exist');
        }
      })['return'](username);
    });
  }

  return promise.then(function activateAccount(user) {
    var userKey = redisKey(user, 'data');

    // set to active
    return redis.pipeline().hget(userKey, 'active').hset(userKey, 'active', 'true').persist(userKey) // WARNING: this is very important, otherwise we will lose user's information in 30 days
    .sadd(config.redis.userSet, user).exec().spread(function pipeResponse(isActive) {
      var status = isActive[1];
      if (status === 'true') {
        throw new Errors.HttpStatusError(413, 'Account ' + user + ' was already activated');
      }
    })['return'](user);
  }).then(function returnUserInfo(user) {
    return jwt.login.call(this, user, audience || config.defaultAudience);
  });
};