'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Promise = require('bluebird');
var redisKey = require('../utils/key.js');

module.exports = function iterateOverActiveUsers(opts) {
  var redis = this.redis;
  var config = this.config;
  var criteria = opts.criteria;
  var audience = opts.audience;
  var filter = opts.filter;

  var strFilter = typeof filter === 'string' ? filter : JSON.stringify(filter || {});
  var order = opts.order || 'ASC';
  var offset = opts.offset || 0;
  var limit = opts.limit || 10;

  return redis.sortedFilteredList(config.redis.userSet, redisKey('*', 'metadata', audience), criteria, order, strFilter, offset, limit).then(function (ids) {
    var length = +ids.pop();
    if (length === 0 || ids.length === 0) {
      return [ids || [], [], length];
    }

    var pipeline = redis.pipeline();
    ids.forEach(function (id) {
      pipeline.hgetall(redisKey(id, 'metadata', audience));
    });
    return Promise.all([ids, pipeline.exec(), length]);
  }).spread(function (ids, props, length) {
    var users = ids.map(function remapData(id, idx) {
      return {
        id: id,
        metadata: _defineProperty({}, audience, props[idx][1])
      };
    });

    return {
      users: users,
      cursor: offset + limit,
      page: Math.floor(offset / limit + 1),
      pages: Math.ceil(length / limit)
    };
  });
};