'use strict';

var ld = require('lodash');
var redisKey = require('../utils/key.js');

/**
 * Updates metadata on a user object
 * @param  {Object} opts
 * @return {Promise}
 */
module.exports = function updateMetadata(opts) {
  var redis = this.redis;
  var username = opts.username;
  var audience = opts.audience;
  var metadata = opts.metadata;

  var metadataKey = redisKey(username, 'metadata', audience);
  var pipeline = redis.pipeline();

  var $remove = metadata.$remove;
  if ($remove && $remove.length > 0) {
    pipeline.hdel(metadataKey, $remove);
  }

  var $set = metadata.$set;
  if ($set && Object.keys($set).length > 0) {
    pipeline.hmset(metadataKey, ld.mapValues($set, JSON.stringify, JSON));
  }

  return pipeline.exec();
};