'use strict';

var Promise = require('bluebird');
var jwt = require('../utils/jwt.js');
var getMetadata = require('./getMetadata.js');

/**
 * Verifies that passed token is signed correctly, returns associated metadata with it
 *
 * @param  {Object}   opts
 * @returns {Promise}
 */
module.exports = function verify(opts) {
  var defaultAudience = this.config.jwt.defaultAudience;
  var token = opts.token;
  var _audience = opts.audience;
  var peek = opts.peek;

  var audience = Array.isArray(_audience) ? _audience : [_audience];

  return jwt.verify.call(this, token, audience, peek).bind(this).then(function decodedToken(decoded) {
    if (audience.indexOf(defaultAudience) === -1) {
      audience.push(defaultAudience);
    }

    // get metadata and return success
    var username = decoded.username;
    return Promise.props({
      username: username,
      metadata: getMetadata.call(this, username, audience)
    });
  });
};