'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x2, _x3, _x4) { var _again = true; _function: while (_again) { var object = _x2, property = _x3, receiver = _x4; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x2 = parent; _x3 = property; _x4 = receiver; _again = true; desc = parent = undefined; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Mservice = require('mservice');
var fs = require('fs');
var path = require('path');
var Mailer = require('ms-mailer-client');
var Promise = require('bluebird');
var Errors = require('common-errors');
var ld = require('lodash');

var _require = require('util');

var fmt = _require.format;

// actions
var register = require('./actions/register.js');
var getMetadata = require('./actions/getMetadata.js');
var updateMetadata = require('./actions/updateMetadata.js');
var challenge = require('./actions/challenge.js');
var activate = require('./actions/activate.js');
var login = require('./actions/login.js');
var logout = require('./actions/logout.js');
var verify = require('./actions/verify.js');
var requestPassword = require('./actions/requestPassword.js');
var updatePassword = require('./actions/updatePassword.js');
var ban = require('./actions/ban.js');
var list = require('./actions/list.js');

// utils
var redisKey = require('./utils/key.js');
var sortedFilteredListLua = fs.readFileSync(path.resolve(__dirname, '../lua/sorted-filtered-list.lua'), 'utf-8');

/**
 * @namespace Users
 */
module.exports = (function (_Mservice) {
  _inherits(Users, _Mservice);

  _createClass(Users, null, [{
    key: 'defaultOpts',

    /**
     * Configuration options for the service
     * @type {Object}
     */
    value: {
      debug: process.env.NODE_ENV === 'development',
      // prefix routes with users.
      prefix: 'users',
      // keep inactive accounts for 30 days
      deleteInactiveAccounts: 30 * 24 * 60 * 60,
      // postfixes for routes that we support
      postfix: {
        // ban, supports both unban/ban actions
        ban: 'ban',

        // challenge. Challenge sends email with a token that is used to activate account
        // often used internally from 'register' method
        challenge: 'challenge',

        // verifies it and activates not banned account
        activate: 'activate',

        // verify token and return metadata
        verify: 'verify',

        // verify credentials and return metadata
        login: 'login',

        // verify token and destroy it
        logout: 'logout',

        // creates new user
        // sends 'challenge', or, if this options is not set, immediately registers user
        // in the future multiple challenge options could be supported, for now it's just an email
        register: 'register',

        // pass metadata out based on username
        // core data only contains username and hashed password
        // this is an application specific part that can store anything here
        getMetadata: 'getMetadata',

        // update metadata based on username
        // rewrites/adds new data, includes both set and remove methods, set overwrites,
        // while remove - deletes. Set precedes over remove and batch updates are supported
        updateMetadata: 'updateMetadata',

        // requests an email to change password
        // can be extended in the future to support more options like secret questions
        // or text messages
        requestPassword: 'requestPassword',

        // updates password - either without any checks or, if challenge token is passed, makes sure it's correct
        updatePassword: 'updatePassword',

        // lists and iterators over registered users
        list: 'list'
      },
      amqp: {
        queue: 'ms-users'
      },
      captcha: {
        secret: 'put-your-real-gcaptcha-secret-here',
        ttl: 3600, // 1 hour - 3600 seconds
        uri: 'https://www.google.com/recaptcha/api/siteverify'
      },
      redis: {
        options: {
          // must have {}, so that the keys end up on a single machine
          keyPrefix: '{ms-users}'
        },
        userSet: 'user-iterator-set'
      },
      jwt: {
        defaultAudience: '*.localhost',
        hashingFunction: 'HS256',
        issuer: 'ms-users',
        secret: 'i-hope-that-you-change-this-long-default-secret-in-your-app',
        ttl: 30 * 24 * 60 * 60 * 1000, // 30 days in ms
        lockAfterAttempts: 5,
        keepLoginAttempts: 60 * 60 },
      // 1 hour
      validation: {
        secret: 'please-replace-this-as-a-long-nice-secret',
        algorithm: 'aes-256-ctr',
        throttle: 2 * 60 * 60, // dont send emails more than once in 2 hours
        ttl: 4 * 60 * 60, // do not let password to be reset with expired codes
        paths: {
          activate: '/activate',
          reset: '/reset'
        },
        subjects: {
          activate: 'Activate your account',
          reset: 'Reset your password'
        },
        senders: {
          activate: 'noreply <support@example.com>',
          reset: 'noreply <support@example.com>'
        },
        email: 'support@example.com'
      },
      server: {
        proto: 'http',
        host: 'localhost',
        port: 8080
      },
      mailer: {
        prefix: 'mailer',
        routes: {
          adhoc: 'adhoc',
          predefined: 'predefined'
        }
      },
      admins: [],
      // enable all plugins
      plugins: ['validator', 'logger', 'amqp', 'redisCluster'],
      // by default only ringBuffer logger is enabled in prod
      logger: process.env.NODE_ENV === 'development',
      // init local schemas
      validator: ['../schemas']
    },

    /**
     * @namespace Users
     * @param  {Object} opts
     * @return {Users}
     */
    enumerable: true
  }]);

  function Users() {
    var _this = this;

    var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Users);

    _get(Object.getPrototypeOf(Users.prototype), 'constructor', this).call(this, ld.merge({}, Users.defaultOpts, opts));

    this.router = function (message, headers, actions, next) {
      var time = process.hrtime();
      var route = headers.routingKey.split('.').pop();
      var defaultRoutes = Users.defaultOpts.postfix;
      var postfix = _this.config.postfix;

      var promise = undefined;
      switch (route) {
        case postfix.verify:
          promise = _this._validate(defaultRoutes.verify, message).then(_this._verify);
          break;
        case postfix.register:
          promise = _this._validate(defaultRoutes.register, message).then(_this._register);
          break;
        case postfix.ban:
          promise = _this._validate(defaultRoutes.ban, message).then(_this._ban);
          break;
        case postfix.challenge:
          promise = _this._validate(defaultRoutes.challenge, message).then(_this._challenge);
          break;
        case postfix.activate:
          promise = _this._validate(defaultRoutes.activate, message).then(_this._activate);
          break;
        case postfix.login:
          promise = _this._validate(defaultRoutes.login, message).then(_this._login);
          break;
        case postfix.logout:
          promise = _this._validate(defaultRoutes.logout, message).then(_this._logout);
          break;
        case postfix.getMetadata:
          promise = _this._validate(defaultRoutes.getMetadata, message).then(_this._getMetadata);
          break;
        case postfix.updateMetadata:
          promise = _this._validate(defaultRoutes.updateMetadata, message).then(_this._updateMetadata);
          break;
        case postfix.requestPassword:
          promise = _this._validate(defaultRoutes.requestPassword, message).then(_this._requestPassword);
          break;
        case postfix.updatePassword:
          promise = _this._validate(defaultRoutes.updatePassword, message).then(_this._updatePassword);
          break;
        case postfix.list:
          promise = _this._validate(defaultRoutes.list, message).then(_this._list);
          break;
        default:
          promise = Promise.reject(new Errors.NotImplementedError(fmt('method "%s"', route)));
          break;
      }

      // if we have an error
      promise = promise['finally'](function auditLog(response) {
        var execTime = process.hrtime(time);
        var meta = {
          message: message,
          headers: headers,
          latency: execTime[0] * 1000 + +(execTime[1] / 1000000).toFixed(3)
        };

        if (response instanceof Error) {
          this.log.error(meta, 'Error performing operation', response);
        } else {
          this.log.info(meta, 'completed operation');
        }
      });

      if (typeof next === 'function') {
        return promise.asCallback(next);
      }

      return promise;
    };

    var config = this.config;

    // map routes we listen to
    var prefix = config.prefix;

    config.amqp.listen = ld.map(config.postfix, function assignPostfix(postfix) {
      return prefix + '.' + postfix;
    });

    var _validateSync = this.validateSync('config', config);

    var error = _validateSync.error;

    if (error) {
      this.log.fatal('Invalid configuration:', error.toJSON());
      throw error;
    }

    this.on('plugin:connect:amqp', function (amqp) {
      _this._mailer = new Mailer(amqp, config.mailer);
    });

    this.on('plugin:close:amqp', function () {
      _this._mailer = null;
    });

    this.on('plugin:connect:redisCluster', function (redis) {
      redis.defineCommand('sortedFilteredList', {
        numberOfKeys: 2,
        lua: sortedFilteredListLua
      });
    });
  }

  /**
   * Getter for mailer client
   * @return {Object}
   */

  _createClass(Users, [{
    key: 'initAdminAccounts',

    /**
     * Initializes Admin accounts
     * @return {Promise}
     */
    value: function initAdminAccounts() {
      var _this2 = this;

      var config = this.config;
      var accounts = config.admins;
      var audience = config.jwt.defaultAudience;
      return Promise.map(accounts, function (account) {
        return register.call(_this2, {
          username: account.username,
          password: account.password,
          audience: audience,
          metadata: {
            firstName: account.firstName,
            lastName: account.lastName,
            roles: ['admin']
          },
          activate: true
        }).reflect();
      }).bind(this).then(function reportStats(users) {
        var totalAccounts = users.length;
        var errors = [];
        var registered = 0;
        users.forEach(function (user) {
          if (user.isFulfilled()) {
            registered++;
          } else {
            errors.push(user.reason());
          }
        });

        this.log.info('Registered admins %d/%d. Errors:', registered, totalAccounts, errors);
      })['finally'](function () {
        _this2.log.info('removing account references from memory');
        config.admins = [];
      });
    }

    /**
     * Router instance, bound to Users module
     * @param  {Object}   message
     * @param  {Object}   headers
     * @param  {Object}   actions
     * @param  {Function} next
     * @return {Promise}
     */
  }, {
    key: '_validate',

    /**
     * @private
     * @param  {String} route
     * @param  {Object} message
     * @return {Promise}
     */
    value: function _validate(route, message) {
      return this.validate(route, message).bind(this)['return'](message)['catch'](function validationError(error) {
        this.log.warn('Validation error:', error.toJSON());
        throw error;
      });
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_list',
    value: function _list(message) {
      return list.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_ban',
    value: function _ban(message) {
      return ban.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_updatePassword',
    value: function _updatePassword(message) {
      return updatePassword.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_requestPassword',
    value: function _requestPassword(message) {
      return requestPassword.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_verify',
    value: function _verify(message) {
      return verify.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_logout',
    value: function _logout(message) {
      return logout.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_login',
    value: function _login(message) {
      return login.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_activate',
    value: function _activate(message) {
      return activate.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_challenge',
    value: function _challenge(message) {
      return challenge.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_register',
    value: function _register(message) {
      return register.call(this, message);
    }

    /**
     * @private
     * @param  {Object} message
     * @return {Promise}
     */
  }, {
    key: '_getMetadata',
    value: function _getMetadata(message) {
      var _this3 = this;

      var username = message.username;

      return this.redis.hexists(redisKey(username, 'data'), 'password').then(function (exists) {
        if (exists !== true) {
          throw new Errors.HttpStatusError(404, '"' + username + '" does not exists');
        }

        return getMetadata.call(_this3, username, message.audience);
      });
    }

    /**
     * @private
     * @param  {Object}  message
     * @return {Promise}
     */
  }, {
    key: '_updateMetadata',
    value: function _updateMetadata(message) {
      var _this4 = this;

      var username = message.username;

      return this.redis.hexists(redisKey(username, 'data'), 'password').then(function (exists) {
        if (exists !== true) {
          throw new Errors.HttpStatusError(404, '"' + username + '" does not exists');
        }

        return updateMetadata.call(_this4, message);
      });
    }
  }, {
    key: 'mailer',
    get: function get() {
      var mailer = this._mailer;
      return mailer ? mailer : this.emit('error', new Errors.NotImplementedError('amqp is not connected'));
    }

    /**
     * Getter for configuration
     * @return {Object}
     */
  }, {
    key: 'config',
    get: function get() {
      return this._config;
    }
  }]);

  return Users;
})(Mservice);