'use strict';

var Errors = require('common-errors');
var Promise = require('bluebird');
var scrypt = require('scrypt');
var bytes = require('bytes');

// setup scrypt
var scryptParams = scrypt.paramsSync(0.1, bytes('32mb'));

exports.hash = function hashPassword(password) {
  return Promise.fromNode(function kdfPromise(next) {
    scrypt.kdf(new Buffer(password, 'utf-8'), scryptParams, next);
  });
};

exports.verify = function verifyPassword(hash, password) {
  return Promise.fromNode(function verifyKdfPromise(next) {
    scrypt.verifyKdf(hash, new Buffer(password, 'utf-8'), next);
  })['catch'](function scruptError(err) {
    throw new Errors.HttpStatusError(403, err.scrypt_err_message || err.message);
  });
};